package org.zalando.intellij.swagger.completion;

import com.intellij.codeInsight.completion.InsertHandler;
import com.intellij.codeInsight.lookup.LookupElement;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import org.zalando.intellij.swagger.completion.field.model.common.Field;
import org.zalando.intellij.swagger.completion.value.model.common.Value;
import org.zalando.intellij.swagger.traversal.Traversal;
import org.zalando.intellij.swagger.traversal.path.openapi.PathResolver;

import java.util.List;
import java.util.Optional;

public class OpenApiCompletionHelper implements CompletionHelper {

    private final PsiElement psiElement;
    private final Traversal traversal;
    private final PathResolver pathResolver;

    public OpenApiCompletionHelper(final PsiElement psiElement,
                                   final Traversal traversal,
                                   final PathResolver pathResolver) {
        this.psiElement = psiElement;
        this.traversal = traversal;
        this.pathResolver = pathResolver;
    }

    public boolean completeRootKey() {
        return pathResolver.childOfRoot(psiElement);
    }

    public boolean completeInfoKey() {
        return pathResolver.childOfInfo(psiElement);
    }

    public boolean completeContactKey() {
        return pathResolver.childOfContact(psiElement);
    }

    public boolean completeLicenseKey() {
        return pathResolver.childOfLicense(psiElement);
    }

    public boolean completePathKey() {
        return pathResolver.childOfPath(psiElement);
    }

    public boolean completeOperationKey() {
        return pathResolver.childOfOperation(psiElement);
    }

    public boolean completeExternalDocsKey() {
        return pathResolver.childOfExternalDocs(psiElement);
    }

    public boolean completeParametersKey() {
        return pathResolver.childOfParameters(psiElement);
    }

    public boolean completeParameterItemsKey() {
        return pathResolver.childOfParameterItems(psiElement);
    }

    public boolean completeResponsesKey() {
        return pathResolver.childOfResponses(psiElement);
    }

    public boolean completeResponseKey() {
        return pathResolver.childOfResponse(psiElement);
    }

    public boolean completeHeaderKey() {
        return pathResolver.childOfHeader(psiElement);
    }

    public boolean completeHeadersKey() {
        return pathResolver.childOfHeaders(psiElement);
    }

    public boolean completeTagKey() {
        return pathResolver.childOfTag(psiElement);
    }

    public boolean completeSecurityDefinitionKey() {
        return pathResolver.childOfSecurityDefinition(psiElement);
    }

    public boolean completeSchemaKey() {
        return pathResolver.childOfSchema(psiElement);
    }

    public boolean completeSchemaItemsKey() {
        return pathResolver.childOfSchemaItems(psiElement);
    }

    public boolean completePropertiesSchemaKey() {
        return pathResolver.childOfPropertiesSchema(psiElement);
    }

    public boolean completeAdditionalPropertiesKey() {
        return pathResolver.childOfAdditionalProperties(psiElement);
    }

    public boolean completeXmlKey() {
        return pathResolver.childOfXml(psiElement);
    }

    public boolean completeDefinitionsKey() {
        return pathResolver.childOfDefinitions(psiElement);
    }

    public boolean completeParameterDefinitionKey() {
        return pathResolver.childOfParameterDefinition(psiElement);
    }

    public boolean completeMimeValue() {
        return pathResolver.isMimeValue(psiElement);
    }

    public boolean completeSchemesValue() {
        return pathResolver.isSchemesValue(psiElement);
    }

    public boolean completeSchemaRefValue() {
        return pathResolver.isSchemaRefValue(psiElement);
    }

    public boolean completeParameterRefValue() {
        return pathResolver.isParameterRefValue(psiElement);
    }

    public boolean completeExampleRefValue() {
        return pathResolver.isExampleRefValue(psiElement);
    }

    public boolean completeRequestBodyRefValue() {
        return pathResolver.isRequestBodyRefValue(psiElement);
    }

    public boolean completeHeaderRefValue() {
        return pathResolver.isHeaderRefValue(psiElement);
    }

    public boolean completeLinkRefValue() {
        return pathResolver.isLinkRefValue(psiElement);
    }

    public boolean completeCallbackRefValue() {
        return pathResolver.isCallbackRefValue(psiElement);
    }

    public List<PsiElement> getChildrenOfArrayObject(final PsiElement psiElement) {
        return traversal.getChildrenOfArrayObject(psiElement);
    }

    public List<String> getTagNames() {
        return traversal.getTagNames(psiElement.getContainingFile());
    }

    public boolean isUniqueKey(final String keyName) {
        return traversal.isUniqueKey(keyName, psiElement);
    }

    public boolean isUniqueArrayStringValue(final String keyName) {
        return traversal.isUniqueArrayStringValue(keyName, psiElement);
    }

    public InsertHandler<LookupElement> createInsertFieldHandler(final Field field) {
        return traversal.createInsertFieldHandler(field);
    }

    public InsertHandler<LookupElement> createInsertValueHandler(final Value value) {
        return traversal.createInsertValueHandler(value);
    }

    public boolean completeBooleanValue() {
        return pathResolver.isBooleanValue(psiElement);
    }

    public boolean completeTypeValue() {
        return pathResolver.isTypeValue(psiElement);
    }

    public boolean completeFormatValue() {
        return pathResolver.isFormatValue(psiElement);
    }

    public boolean completeStyleValue() {
        return pathResolver.isStyleValue(psiElement);
    }

    public boolean completeInValue() {
        return pathResolver.isInValue(psiElement);
    }

    public boolean completeResponseRefValue() {
        return pathResolver.isResponseRefValue(psiElement);
    }

    public boolean completeResponseDefinition() {
        return pathResolver.childOfResponseDefinition(psiElement);
    }

    public boolean completeRootSecurityKey() {
        return pathResolver.childOfRootSecurityKey(psiElement);
    }

    public boolean completeOperationSecurityKey() {
        return pathResolver.childOfOperationSecurityKey(psiElement);
    }

    public Optional<String> extractSecurityNameFromSecurityObject(final PsiElement psiElement) {
        return traversal.extractSecurityNameFromSecurityItem(psiElement);
    }

    public List<String> getSecurityScopesIfOAuth2(final PsiElement securityDefinitionItem) {
        return traversal.getSecurityScopesIfOAuth2(securityDefinitionItem);
    }

    public boolean completeSecurityScopeNameValue() {
        return pathResolver.isSecurityScopeNameValue(psiElement);
    }

    public Optional<String> getKeyNameOfObject(final PsiElement psiElement) {
        return traversal.getKeyNameOfObject(psiElement);
    }

    public PsiFile getPsiFile() {
        return psiElement.getContainingFile();
    }

    public boolean completeDefinitionRefValue() {
        return false;
    }

    public Optional<String> getParentKeyName() {
        return traversal.getParentKeyName(psiElement);
    }

    public Optional<PsiElement> getParentByName(final String parentName) {
        return traversal.getParentByName(psiElement, parentName);
    }

    public boolean completeItemsCollectionFormat() {
        return pathResolver.childOfItemsCollectionFormat(psiElement);
    }

    public boolean completeParametersCollectionFormat() {
        return pathResolver.childOfParametersCollectionFormat(psiElement);
    }

    public boolean completeHeadersCollectionFormat() {
        return pathResolver.childOfHeadersCollectionFormat(psiElement);
    }

    public boolean completeTagsValue() {
        return pathResolver.isTagsValue(psiElement);
    }

    public boolean completeServerKey() {
        return pathResolver.childOfServer(psiElement);
    }

    public boolean completeServerVariableKey() {
        return pathResolver.childOfServerVariable(psiElement);
    }

    public boolean completeComponentKey() {
        return pathResolver.childOfComponent(psiElement);
    }

    public boolean completeRequestBodyKey() {
        return pathResolver.childOfRequestBody(psiElement);
    }

    public boolean completeMediaTypeKey() {
        return pathResolver.childOfMediaType(psiElement);
    }

    public boolean completeExampleKey() {
        return pathResolver.childOfExample(psiElement);
    }

    public boolean completeEncodingKey() {
        return pathResolver.childOfEncoding(psiElement);
    }

    public boolean completeLinkKey() {
        return pathResolver.childOfLink(psiElement);
    }

    public boolean completeCallbackKey() {
        return pathResolver.childOfCallback(psiElement);
    }

    public boolean completeSecuritySchemeKey() {
        return pathResolver.childOfSecurityScheme(psiElement);
    }

    public boolean completeContentKey() {
        return pathResolver.childOfContent(psiElement);
    }
}
